@@ -10,6 +10,7 @@
 #include "call_interfaces.h"
 #include "rtplib.h"
 #include "ice.h"
+#include "log_funcs.h"
 
 
 struct janus_session { // "login" session
@@ -1205,6 +1206,8 @@ static const char *janus_create(JsonReader *reader, JsonBuilder *builder, struct
 	json_builder_add_int_value(builder, session_id);
 	json_builder_end_object(builder); // }
 
+	obj_put(session);
+
 	return NULL;
 }
 
@@ -1885,6 +1888,8 @@ err:
 	if (session)
 		obj_put(session);
 
+	log_info_reset();
+
 	return NULL;
 }
 
